print("Timurids Scripts")

include("FLuaVector.lua")

function PositionCalculator(i1, i2)
	return HexToWorld(ToHexFromGrid(Vector2(i1, i2)))
end

local bTimDummy = GameInfoTypes.BUILDING_TIMURID_MOD_DUMMY
local TimuridsID = GameInfoTypes.CIVILIZATION_TIMURIDS_MOD
local L = Locale.ConvertTextKey
local tBuildings = {}
local tWaterBuildings = {}

-- Trade Route Yields -- adds dummy buildings to Registan, based on number of International Trade Routes in/out of the Timurid Capital.
function MadrasahTradeYields(PlayerID)
	local pPlayer = Players[PlayerID]

	if not pPlayer:GetCivilizationType() == TimuridsID then return end
	
	local eRegistan = GameInfoTypes.BUILDING_TIMURID_MADRASAH_MOD
	if (pPlayer:CountNumBuildings(eRegistan) == 0) then return end

	local pCapital = pPlayer:GetCapitalCity()
	local tTradeRoutes = pPlayer:GetTradeRoutes()
	local iCounter = 0

	for i, tradeRoute in ipairs(tTradeRoutes) do
		if (tradeRoute.FromCity:IsHasBuilding(eRegistan) and tradeRoute.ToCivilizationType ~= TimuridsID) or (tradeRoute.ToCity:IsHasBuilding(eRegistan) and tradeRoute.FromCivilizationType ~= TimuridsID) then
			iCounter = iCounter + 1
		end
	end
	
	if pCapital then
		if pCapital:IsHasBuilding(GameInfoTypes.BUILDING_TIMURID_MADRASAH_1_MOD) then
			pCapital:SetNumRealBuilding(GameInfoTypes.BUILDING_TIMURID_MADRASAH_1_DUMMY, iCounter)
		end
		if pCapital:IsHasBuilding(GameInfoTypes.BUILDING_TIMURID_MADRASAH_2_MOD) then
			pCapital:SetNumRealBuilding(GameInfoTypes.BUILDING_TIMURID_MADRASAH_2_DUMMY, iCounter)
		end
		if pCapital:IsHasBuilding(GameInfoTypes.BUILDING_TIMURID_MADRASAH_3_MOD) then
			pCapital:SetNumRealBuilding(GameInfoTypes.BUILDING_TIMURID_MADRASAH_3_DUMMY, iCounter)
		end
	end
end

-- Jewel of the East UA of Timurids - all conquered buildings are immediately built in capital; consist of 2 functions to achieve UBs; yields on conquer if not conquered before
function JewelOfTheEast1(iPlayer, iUnitOrCity, iRole, bIsCity)
    if not (bIsCity and iRole == 1) then return end
 
    local pPlayer = Players[iPlayer]
 
    if not pPlayer then return end
 
    local pCity = pPlayer:GetCityByID(iUnitOrCity)
    tBuildings = {}
    tWaterBuildings = {}
  
    for building in GameInfo.Buildings() do
        if pCity:IsHasBuilding(building.ID) and (not building.NukeImmune or building.ID == GameInfoTypes.BUILDING_MILITARY_BASE) and building.HurryCostModifier ~= -1 and building.CapitalOnly ~= 1 then -- no wonders, dummies, or tradition policy buildings
            local bIsUnique = false
            local bIsFound = false
         
            for uniqueBuilding in DB.Query("select * from Civilization_BuildingClassOverrides where BuildingType = ?", building.Type) do
                bIsUnique = true
             
                for equivalentBuilding in GameInfo.Buildings() do
                    if equivalentBuilding.BuildingClass == uniqueBuilding.BuildingClassType then
                        local bIsBaseBuilding = true
                     
                        for equivalentUniqueBuilding in DB.Query("select * from Civilization_BuildingClassOverrides where BuildingType = ?", equivalentBuilding.Type) do
                            bIsBaseBuilding = false
                        end
                     
                        if bIsBaseBuilding then
                            bIsFound = true
                         
                            if equivalentBuilding.Water then
                                table.insert(tWaterBuildings, equivalentBuilding)
                            else
                                table.insert(tBuildings, equivalentBuilding)
                            end
                         
                            break
                        end
                    end
                end
             
                if bIsFound then
                    break
                end
            end
         
            if not bIsUnique then
                if building.Water then
                    table.insert(tWaterBuildings, building)
                else
                    table.insert(tBuildings, building)
                end
            end        
        end
    end
end


function JewelOfTheEast2(iOldOwner, bIsCapital, iX, iY, iNewOwner, iPop, bConquest)
    -- buildings part
    local pPlayer = Players[iNewOwner]
    local pCapital = pPlayer:GetCapitalCity()
    local pCapitalX, pCapitalY, pCapitalID = pCapital:GetX(), pCapital:GetY(), pCapital:GetID()
    local sName = pCapital:GetName()
    local iCoastalBuildings = 0
 
    if not (pPlayer and pPlayer:GetCivilizationType() == TimuridsID) then return end

    -- yields part - on City Capture, Track the number of population in the City
    local pCity = Map.GetPlot(iX, iY):GetPlotCity()
	pCity:SetNumRealBuilding(GameInfoTypes.BUILDING_UNIQUE_RAZE_DUMMY, pCity:GetPopulation())
    
    if pCapital:IsCoastal() then
        iCoastalBuildings = #tWaterBuildings
    end
 
    local iNumberOfBuildings = #tBuildings + iCoastalBuildings
 
    if iNumberOfBuildings == 0 then return end
     
    local sCitiesNames = ""
 
    for i, building in pairs(tBuildings) do
        if not pCapital:IsHasBuilding(building.ID) then        
            pCapital:SetNumRealBuilding(building.ID, 1)
         
            if i - iCoastalBuildings == iNumberOfBuildings then
                sCitiesNames = sCitiesNames .. " and " .. L(building.Description)
            elseif sCitiesNames == "" then
                sCitiesNames = sCitiesNames .. L(building.Description)
            else
                sCitiesNames = sCitiesNames .. ", " .. L(building.Description)
            end
        end
    end
 
    if pCapital:IsCoastal() then
        for i, building in pairs(tWaterBuildings) do
            if not pCapital:IsHasBuilding(building.ID) then        
                pCapital:SetNumRealBuilding(building.ID, 1)
             
                if i == #tWaterBuildings then
                    sCitiesNames = sCitiesNames .. " and " .. L(building.Description)
                elseif sCitiesNames == "" then
                    sCitiesNames = sCitiesNames .. L(building.Description)
                else
                    sCitiesNames = sCitiesNames .. ", " .. L(building.Description)
                end
            end
        end
    end
 
    if pPlayer:IsHuman() and pPlayer:IsTurnActive() then
        local sText = "Foreign architects have constructed following buildings in your [ICON_CAPITAL] Capital: " .. sCitiesNames .. "."
        local sTitle = "Jewel of the East"
     
        pPlayer:AddNotification(NotificationTypes.NOTIFICATION_WONDER_COMPLETED,
            sText,
            sTitle,
            pCapitalX, pCapitalY, GameInfoTypes.BUILDING_PALACE, -1)
    end
end

-- Permanent yields on city raze -- Adds yields via dummy building to Capital every time Timurids destroy a city
function CapitalBonusOnDestroy(iPlayer, iX, iY)
	local pPlayer = Players[iPlayer]
	
	if pPlayer:GetCivilizationType() == TimuridsID then
		local pCity = Map.GetPlot(iX, iY):GetPlotCity()
		local pCapital = pPlayer:GetCapitalCity()
		local iRazeBonus = (math.ceil(pCity:GetNumRealBuilding(GameInfoTypes.BUILDING_UNIQUE_RAZE_DUMMY) / 2))
		pCapital:SetNumRealBuilding(bTimDummy, (pCapital:GetNumRealBuilding(bTimDummy) + iRazeBonus))
		
		if pPlayer:IsHuman() and pPlayer:IsTurnActive() then
			local sName = pCapital:GetName()
			local pCapitalX, pCapitalY, pCapitalID = pCapital:GetX(), pCapital:GetY(), pCapital:GetID()
			local vCityPosition = PositionCalculator(pCapitalX, pCapitalY)
			
			pPlayer:AddNotification(NotificationTypes.NOTIFICATION_INSTANT_YIELD,
				'Cities burn before your might. [COLOR_POSITIVE_TEXT]'..sName..'[ENDCOLOR] has gained +'..iRazeBonus..' to all yields',
				'Artisans fled to '..sName,
				pCapitalX, pCapitalY, pCapitalID)
		end
	end
end

if Game.IsCivEverActive(TimuridsID) then
	GameEvents.CityRazed.Add(CapitalBonusOnDestroy)
	GameEvents.BattleJoined.Add(JewelOfTheEast1)
	GameEvents.CityCaptureComplete.Add(JewelOfTheEast2)
	GameEvents.PlayerDoTurn.Add(MadrasahTradeYields)
end